
#include <stdio.h>
#include <stdlib.h>
#include "curl/curl.h"
#include "make_header.h"
#include "util.h"
#include "sign.h"
#include "file_deal.h"
#include "xml_util.h"
#include "hmacsha256.h"


static void set_method(MethodType method_type, void* handler);
static char* compute(const char* sign, const char* secret_key, char* b64);
static char* make_url(const char* host, const char* bucket_name,
    const char* object, const char* query_args, char* url);
    
static int make_header_common(const request_options_t* options, MethodType method_type,
    const char* bucket, const char* object, const char* data, int buf_len,
    const char* query_args, const char* headers_str, OpType op_type, buffer* resp);

static void set_method(MethodType method_type, void* handler) {
	switch(method_type) {
	case PUT_METHOD:
		curl_easy_setopt(handler, CURLOPT_UPLOAD, 1L);
		curl_easy_setopt(handler, CURLOPT_PUT, 1L);
		break;
	case DELETE_METHOD:
		curl_easy_setopt(handler, CURLOPT_CUSTOMREQUEST, "DELETE");
        break;
    case POST_METHOD:
        curl_easy_setopt(handler, CURLOPT_POST, 1L);
		break;
	case HEAD_METHOD:
		curl_easy_setopt(handler, CURLOPT_NOBODY, 1L);
        break;
    default:
        break;
	}
}

static char* compute(const char* sign, const char* secret_key, char* b64) {
	unsigned char hmac[20];
	int b64Len;
	HMAC_SHA1(hmac, (const unsigned char *)secret_key,
		strlen(secret_key), (const unsigned char *)sign, strlen(sign));
	b64Len = base64Encode(hmac, 20, b64);
	b64[b64Len] = '\0';
	return b64;
}

static char* make_url(const char* host, const char* bucket_name,
	const char* object, const char* query_args, char* url) {
	if (host != NULL) {
	    strcat(url, host);
	}
    
    if (bucket_name != NULL && strlen(bucket_name) >0) {
        strcat(url, "/");
        strcat(url, bucket_name);
        if (object != NULL && strlen(object) > 0) {
            strcat(url, "/");
            strcat(url, object);
        }
    }
    if (query_args != NULL && strlen(query_args) > 0) {
        strcat(url, "?");
        strcat(url, query_args);
    }
    
	strcat(url, "\0");
	return url;
}

static const char* HTTP = "HTTP";
static const char* CONTENT_LENGTH= "Content-Length";
static const char* ETAG ="ETag";
static void parse_http_header(CURL* handler, buffer* resp,oos_pool_t *p) {
	char* token = NULL;
	char* out_ptr = NULL;
	char header_copy[2010] = { '\0' };
	char content_len_str[100] = { '\0' };
	char* first_space = NULL;
	char* sec_space = NULL;

    if (handler == NULL || resp == NULL) {
        return;
    }
    // 1. get http status code
    curl_easy_getinfo(handler, CURLINFO_HTTP_CODE, &resp->status_code);
    if (resp->header == NULL) {
        return;
    }
    // 2. get http status msg
	strncpy(header_copy, resp->header, strlen(resp->header));
	// 2.1 get first line: start with 'HTTP'
#ifdef _WIN32
	token = strtok_s(header_copy, "\r\n", &out_ptr);    
#endif
#ifdef __linux__
	token = mystrtok_r(header_copy, "\r\n", &out_ptr);
#endif
    while (token != NULL) {
        char * pLineTmp = NULL, *pKeyTmp, *line = NULL;
        char *key = NULL, * value = NULL;
        char *pbuf = oos_pcalloc(p, strlen(token)+1);
        strcpy(pbuf, token);

        while(NULL != (line = strtok_r(pbuf, "\r\n", &pLineTmp)))
        {
            pbuf = NULL;
            key = strtok_r(line, ":", &pKeyTmp);
            if(key){
               value = strtok_r(NULL, ":", &pKeyTmp);
               if(value){
                    oos_table_set(resp->headers, key, value+1);
               }
            }
        }
        if (strncmp(HTTP, token, strlen(HTTP)) == 0) {
            // 2.2 split "HTTP/xxxx status_code status_msg" by space
            first_space = strstr(token, " ");
            if (first_space == NULL) {
                return;
            }
            sec_space = strstr(first_space + 1, " ");
            // 2.3 copy status msg
            strcat(resp->status_msg, sec_space + 1);
            strcpy(resp->status_msg, sec_space + 1);
        } else if (strncmp(CONTENT_LENGTH, token, strlen(CONTENT_LENGTH)) == 0) {
                first_space = strstr(token, ": ");
                if (first_space == NULL) {
                    return;
                }   
                strcat(content_len_str, first_space + 2); 
                if (strlen(content_len_str) > 0) {
                    resp->content_length = strtol(content_len_str, NULL, 10);
                }
            break;
        }  else if (strncmp(ETAG, token, strlen(ETAG)) == 0) {
                first_space = strstr(token, ": ");
                if (first_space == NULL) {
                    return;
                } 
                strncpy(resp->etag, first_space + 3, 32);
        }
        first_space = NULL;
        sec_space = NULL;
#ifdef _WIN32
        token = strtok_s(NULL, "\r\n", &out_ptr);    
#endif
#ifdef __linux__
        token = mystrtok_r(NULL, "\r\n", &out_ptr);
#endif
    }
}

static FILE* up_file_preprocess(MethodType method_type, const char* data,
        CURL* handler, buffer* resp) {
    FILE* file = NULL;
    size_t file_size;
    if (method_type == PUT_METHOD) {
        file = fopen(data, "rb");
        if (file == NULL) {
            fprintf(stderr, "open file %s failed : %m\n", data);
            return file;
        }
        fseek(file, 0, SEEK_END);
        file_size = ftell(file);
        fseek(file, 0, SEEK_SET);
        file_up(handler, file, file_size, resp);
    } else if (method_type == GET_METHOD) {
        file = fopen(data, "wb");
        if (file == NULL) {
            return file;
        }
        file_down(handler, file, resp);
    }
    return file;
}

static void buf_preprocess(MethodType method_type, BufData* data,
        CURL* handler, buffer* resp) {
    if (method_type == PUT_METHOD
            || method_type == POST_METHOD) {
        buf_up(handler, data, data->len, resp);
    } else if (method_type == GET_METHOD) {
        buf_down(handler, resp);
    }
}

static void buf_preprocess_segment(MethodType method_type, BufData* data,
                    CURL* handler, buffer* resp, void *callback) {
    if (method_type == PUT_METHOD) {
        buf_up(handler, data, data->len, resp);
    } else if (method_type == GET_METHOD) {
        buf_down_segment(handler, resp, callback);
    }
}
static void make_V4_Authorization(const request_options_t* options, const char* bucket, const char* object, const char* headers
                                  ,const char* data, int buf_len, OpType op_type, MethodType method_type, const char* query_args,char* url_encode
                                  ,char* auth, char* date_t, char* gtm, char* x_amz_date)
{
    char* host = (char *)options->endpoint.data;
    char* access_key = (char *)options->access_key.data;
    char* secret_key = (char *)options->secret_key.data;

    char utc_time[64+1] = {"\0"};
    char region[256+1]  = {"\0"};
    char service[256+1] = {"\0"};

    get_region(data, host ,region);
    get_service(data, service);
    get_utc_time(utc_time);
    get_date(date_t);

    //strcpy(utc_time, "20190301T104334Z");
    //strcpy(date_t, "20190301");


    strcat(x_amz_date, "x-amz-Date: ");
    strcat(x_amz_date, utc_time);
    if(strlen(headers) > 0){
        strcat(headers, "\n");
    }
    strcat(headers, x_amz_date);

    // add x-amz-content-sha256
    if(strstr(headers, "x-amz-content-sha256") == NULL){
        strcat(headers, "\n");

        if(OOS_TRUE == options->is_sign_payload){
            if(buf_len > 0){
			    char sha256[32 + 1] = { "\0" };
				sha2((unsigned char*)data, buf_len, (unsigned char*)sha256, 0);
				char hex_256[64 + 1] = { "\0" };
				HexToStr(hex_256, sha256, 32);
				strcat(headers, "x-amz-content-sha256: ");
				strcat(headers, hex_256);
			}else if (FILE_OP == op_type && strlen(data) > 0){
				char sha256[32 + 1] = { "\0" };
				sha2_file((const char*)data, (unsigned char*)sha256, 0);
				char hex_256[64 + 1] = { "\0" };
				HexToStr(hex_256, sha256, 32);
				strcat(headers, "x-amz-content-sha256: ");
				strcat(headers, hex_256);
            }else{
               strcat(headers, "x-amz-content-sha256: e3b0c44298fc1c149afbf4c8996fb92427ae41e4649b934ca495991b7852b855");
            }
        }else{
           strcat(headers, "x-amz-content-sha256: UNSIGNED-PAYLOAD");
        }
    }

    // add content-length
    if(buf_len > 0){
        strcat(headers, "\n");
        char content_length[1024] = {"\0"};
        sprintf(content_length, "Content-Length: %d", buf_len);

        strcat(headers, content_length);
        printf("len:%d\n", strlen(headers));
    }

    char headers_deal[2048]={"\0"};
    strcpy(headers_deal, headers);

    // add HOST
    strcat(headers_deal, "\n");
    strcat(headers_deal, "HOST: ");
    strcat(headers_deal, host);

    printf("header len:%d \n", strlen(headers));

    // make url
    char url[1024+1] = { '\0' };
    make_url(host, bucket, object, query_args, url);
    //strcat(url, "sdk-c-test1.oos-cn.ctyunapi.cn/copypartdst.txt");
    urlEncode(url_encode, url, strlen(url));


    // 1 canned request
    char canoncal_request[2048+1] = {"\0"};
    char signed_headers[1024+1] = {"\0"};
    make_canonicalized_request_V4(method_type, bucket, object, query_args, headers_deal, canoncal_request, signed_headers);
    printf("===============================================================================================\n");
    printf("canonical request:%s\n", canoncal_request);
    printf("===============================================================================================\n");

    // 2 make sign string
    char string_to_sign[4096+1] = {"\0"};
    make_string_to_sign_V4(utc_time, date_t, canoncal_request, region, service,  string_to_sign);
    printf("signed string:%s\n", string_to_sign);
    printf("===============================================================================================\n");

    // 3 make sign secret key ************************************************/
    // 3.1 date key
    char date_key[32+1] = {"\0"};
    char   aws_secret_key[128+1] = {"\0"};
    strcat(aws_secret_key, "AWS4");
    strcat(aws_secret_key, secret_key);
    sha2_hmac((unsigned char*)aws_secret_key, strlen(aws_secret_key), (unsigned char*)date_t, strlen(date_t), (unsigned char*)date_key, 0);

    // 3.2 date region key
    char region_key[32+1] = {"\0"};
    sha2_hmac((unsigned char*)date_key, strlen(date_key), (unsigned char*)region, strlen(region), (unsigned char*)region_key, 0);

    // 3.3 date_region_service_key
    char service_key[1024] = {"\0"};
    sha2_hmac((unsigned char*)region_key, 32, (unsigned char*)service, strlen(service), (unsigned char*)service_key, 0);

    // 3.4 sign key
    char sign_key[32+1] = {"\0"};
    sha2_hmac((unsigned char*)service_key, 32, (unsigned char*)"aws4_request", strlen("aws4_request"), (unsigned char*)sign_key, 0);

    // 4 make signature
    char signature[32+1] = {"\0"};
    sha2_hmac((unsigned char*)sign_key, 32, (unsigned char*)string_to_sign, strlen(string_to_sign), (unsigned char*)signature, 0);

    // 5 makse authorization
    strcat(auth, "Authorization: ");
    strcat(auth, "AWS4-HMAC-SHA256 ");

    // Credentical
    strcat(auth, "Credential=");
    strcat(auth, access_key);
    strcat(auth, "/");
    strcat(auth, date_t);
    strcat(auth, "/");
    strcat(auth, region);
    strcat(auth, "/");
    strcat(auth, service);
    strcat(auth, "/");
    strcat(auth, "aws4_request");
    strcat(auth, ",");

    // signheaders
    strcat(auth, " SignedHeaders=");
    strcat(auth, signed_headers);
    strcat(auth, ",");

    // signature
    strcat(auth, " Signature=");
    char hex_signature[64+1] = {"\0"};
    HexToStr(hex_signature, signature, 32);
    strcat(auth, hex_signature);
    printf(" len:%d authorization:%s\n", strlen(auth), auth);
    printf("===============================================================================================\n");
}
static void make_V4_Authorization_presigned_url(const request_options_t* options, const char* bucket, const char* object, const char* headers
                                  ,const char* data, int buf_len, MethodType method_type, const char* query_args,char* url_encode
                                  ,char* auth, char* date_t, char* gtm, char* x_amz_date, const int expire_sec)
{
    char* host = (char *)options->endpoint.data;
    char* access_key = (char *)options->access_key.data;
    char* secret_key = (char *)options->secret_key.data;


    char utc_time[64+1] = {"\0"};
    char region[256+1]  = {"\0"};
    char service[256+1] = {"\0"};

    get_region(data, host ,region);
    get_service(data, service);
    get_utc_time(utc_time);
    get_date(date_t);

    //strcpy(utc_time, "20190226T140706Z");
    //strcpy(date_t, "20190226");

    // make url
    char url[1024+1] = { '\0' };
    make_url(host, bucket, object, query_args, url);
    urlEncode(url_encode, url, strlen(url));

    if(strlen(query_args) > 0){
        strcat(query_args, "&");
    }

    // add x-amz-expires
    char expires[128+1] = {"\0"};
    snprintf(expires, sizeof(expires), "X-Amz-Expires=%d", expire_sec);
    strcat(query_args, expires);
    strcat(query_args, "&");

    // add x-amz-algorithm to query_param
    char algorithm[128] = {"\0"};
    strcat(algorithm, "X-Amz-Algorithm=AWS4-HMAC-SHA256");
    strcat(query_args, algorithm);
    strcat(query_args, "&");

    // add Credentical to query_param
     char Credentical[512+1] = {"\0"};
    strcat(Credentical, "X-Amz-Credential=");
    strcat(Credentical, access_key);
    strcat(Credentical, "/");
    strcat(Credentical, date_t);
    strcat(Credentical, "/");
    strcat(Credentical, region);
    strcat(Credentical, "/");
    strcat(Credentical, service);
    strcat(Credentical, "/");
    strcat(Credentical, "aws4_request");
    strcat(query_args, Credentical);
    strcat(query_args, "&");

    // add x-amz-data to query_param
    strcat(x_amz_date, "X-Amz-Date=");
    strcat(x_amz_date, utc_time);
    strcat(query_args, x_amz_date);

    char headers_deal[1024]={"\0"};
    strcpy(headers_deal, headers);

    // add HOST to header
    strcat(headers_deal, "\n");
    strcat(headers_deal, "HOST: ");
    strcat(headers_deal, host);

    // 1 canned request
    char canoncal_request[2048+1] = {"\0"};
    char signed_headers[1024+1] = {"\0"};
    make_canonicalized_request_V4_presigned_url(method_type, bucket, object, query_args, headers_deal, canoncal_request, signed_headers);
    printf("===============================================================================================\n");
    printf("canonical request:%s\n", canoncal_request);
    printf("===============================================================================================\n");

    // 2 make sign string
    char string_to_sign[4096+1] = {"\0"};
    make_string_to_sign_V4(utc_time, date_t, canoncal_request, region, service,  string_to_sign);
    printf("signed string:%s\n", string_to_sign);
    printf("===============================================================================================\n");

    // 3 make sign secret key ************************************************/
    // 3.1 date key
    char date_key[32+1] = {"\0"};
    char aws_secret_key[128+1] = {"\0"};
    strcat(aws_secret_key, "AWS4");
    strcat(aws_secret_key, secret_key);
    sha2_hmac((unsigned char*)aws_secret_key, strlen(aws_secret_key), (unsigned char*)date_t, strlen(date_t), (unsigned char*)date_key, 0);

    // 3.2 date region key
    char region_key[32+1] = {"\0"};
    sha2_hmac((unsigned char*)date_key, strlen(date_key), (unsigned char*)region, strlen(region), (unsigned char*)region_key, 0);

    // 3.3 date_region_service_key
    char service_key[32+1] = {"\0"};
    sha2_hmac((unsigned char*)region_key, 32, (unsigned char*)service, strlen(service), (unsigned char*)service_key, 0);

    // 3.4 sign key
    char sign_key[32+1] = {"\0"};
    sha2_hmac((unsigned char*)service_key, 32, (unsigned char*)"aws4_request", strlen("aws4_request"), (unsigned char*)sign_key, 0);

    // 4 make signature
    char signature[32+1] = {"\0"};
    sha2_hmac((unsigned char*)sign_key, 32, (unsigned char*)string_to_sign, strlen(string_to_sign), (unsigned char*)signature, 0);

    //================================================================================================
    // 5 make return url
    //================================================================================================
    char auth_no_encode[2048+1] = {"\0"};

    // x-amz-algorithm
    strcat(auth_no_encode, algorithm);
    strcat(auth_no_encode, "&");

    // x-amz-expires
    strcat(auth_no_encode, expires);
    strcat(auth_no_encode, "&");

    // x-amz-Credentical
    strcat(auth_no_encode, Credentical);
    strcat(auth_no_encode, "&");

    // x-amz-SignedHeaders
    strcat(auth_no_encode, "&X-Amz-SignedHeaders=");
    strcat(auth_no_encode, signed_headers);
    strcat(auth_no_encode, "&");

    // x-amz-date
    strcat(auth_no_encode, x_amz_date);
    strcat(auth_no_encode, "&");

    // x-amz-signature
    strcat(auth_no_encode, "X-Amz-Signature=");

    char hex_signature[64+1] = {"\0"};
    HexToStr(hex_signature, signature, 32);
    strcat(auth_no_encode, hex_signature);

    // encode
    char encode_auth[2048] = {"\0"};
    make_canon_query_param_V4_presigned_url(auth_no_encode, encode_auth);

    strcat(auth , url);
    strcat(auth, "?");
    strcat(auth, encode_auth);

    printf("authorization:%s\n", auth);
    printf("===============================================================================================\n");
}

static void make_V2_Authorization(const request_options_t* options, const char* bucket, const char* object, const char* headers
                          ,MethodType method_type, const char* query_args,char* url_encode
                          ,char* auth, char* date_t, char* gtm)
{
    char* host = (char *)options->endpoint.data;
    char* access_key = (char *)options->access_key.data;
    char* secret_key = (char *)options->secret_key.data;

    char origin_sign[1024] = { '\0' };
    char url[1024] = { '\0' };
    char b64[((20 + 1) * 4) / 3+1] = { '\0' };

    // 1. make url
    make_url(host, bucket, object, query_args, url);
    //char url_encode[1024] = {0};
    urlEncode(url_encode, url, strlen(url));
    // 2. get time
    get_time(date_t);
    //strcpy(date_t,"Wed, 23 Aug 2017 01:52:23 GMT");
    strcat(gtm, "Date: ");
    strcat(gtm, date_t);
    // 3. make origin sign
    char *encode_bucket = bucket, *encode_object = object, *encode_query_args = query_args;
    if(bucket){
       encode_bucket = oos_palloc(options->pool, strlen(bucket)*3);
       urlEncode(encode_bucket, bucket, strlen(bucket));
    }
    if(object){
        encode_object = oos_palloc(options->pool, strlen(object)*3);
        urlEncode(encode_object, object, strlen(object));
    }
    if(query_args){
        encode_query_args = oos_palloc(options->pool, strlen(query_args)*3);
        urlEncode(encode_query_args, query_args, strlen(query_args));
    }
    make_origin_sign(method_type, encode_bucket, encode_object,
        encode_query_args, headers, date_t, origin_sign);
    // make_origin_sign(method_type, bucket, object,
    // 	query_args, headers, date_t, origin_sign);

    char actual_header[1024] = { '\0' };
    char src_header[100] = { '\0' };
    char  base64_buf[128];
    //snprintf(origin_sign, 1024, "1506180062\n\n/mysecondbucket-10000/livetest");
    compute_buf_md5b64(origin_sign, strlen(origin_sign), base64_buf);
    //strcpy(base64_buf,"Wed, 23 Aug 2017 01:52:23 GMT");
    snprintf(actual_header, 1024, "Content-Md5: %s\n", base64_buf);
    // printf( "Content-Md5: %s\n", base64_buf);
    strcat(src_header, "x-amz-accessKey: ");
    if (access_key != NULL) {
        strcat(src_header, access_key);
    }
    strcat(actual_header, src_header);
    printf("url_encode is %s\n",url_encode);
    printf("url is %s\n",url);
    printf("---- string  to sign--:\n%s\n---end---\n",origin_sign);
    // 4. compute base64_encoded sign
    strcat(auth, "Authorization: AWS ");
    strcat(auth, access_key);
    strcat(auth, ":");
    strcat(auth, compute(origin_sign, secret_key, b64));
    // printf("mysignature:\n%s\n",b64);
    // printf("signature:\n%s\n",auth);
}

static void make_V2_Authorization_presigned_url(const request_options_t* options, const char* bucket, const char* object, const char* headers
                          ,MethodType method_type, const char* query_args,char* url_encode
                          ,char* auth, char* date_t, char* gtm, const int expirs_sec)
{
    char* host = (char *)options->endpoint.data;
    char* access_key = (char *)options->access_key.data;
    char* secret_key = (char *)options->secret_key.data;

    char origin_sign[1024] = { '\0' };
    char url[1024] = { '\0' };
    char b64[((20 + 1) * 4) / 3+1] = { '\0' };

    // 1. make url
    make_url(host, bucket, object, query_args, url);
    urlEncode(url_encode, url, strlen(url));
    // 2. get time
    get_utc_time_expires(date_t, 3600);
    //strcpy(date_t,"1551249120");


    // 3. make origin sign
    char *encode_bucket = bucket, *encode_object = object, *encode_query_args = query_args;
    if(bucket){
       encode_bucket = oos_palloc(options->pool, strlen(bucket)*3);
       urlEncode(encode_bucket, bucket, strlen(bucket));
    }
    if(object){
        encode_object = oos_palloc(options->pool, strlen(object)*3);
        urlEncode(encode_object, object, strlen(object));
    }

    if(query_args){
        encode_query_args = oos_palloc(options->pool, strlen(query_args)*3);
        urlEncode(encode_query_args, query_args, strlen(query_args));
    }
    make_origin_sign(method_type, encode_bucket, encode_object,
        encode_query_args, headers, date_t, origin_sign);

    char  base64_buf[128];
    compute_buf_md5b64(origin_sign, strlen(origin_sign), base64_buf);

    // 4. compute base64_encoded sign
    strcat(auth, url_encode);
    strcat(auth, "?");

    strcat(auth, "AWSAccessKeyId=");
    strcat(auth, access_key);
    strcat(auth, "&");

    strcat(auth, "Expires=");
    char expire_sec_string[64] = {"\0"};
    sprintf(expire_sec_string, "%d", expirs_sec);
    strcat(auth, date_t);
    strcat(auth, "&");

    strcat(auth, "Signature=");

    char* signature = compute(origin_sign, secret_key, b64);
    char* encode_signature = oos_palloc(options->pool, strlen(signature)*3);
    urlEncode_presigned_url(encode_signature, signature, strlen(signature));

    strcat(auth, encode_signature);
}


static int make_header_common(const request_options_t* options, MethodType method_type,
	const char* bucket, const char* object, const char* data, int buf_len,
	const char* query_args, const char* headers_str, OpType op_type, buffer* resp) {
    char* headers = (char*)headers_str;

	FILE *file = NULL;
    char url_encode[1024] = {0};
    char auth[1024] = { '\0' };
    char date_t[64] = { '\0' };
    char x_amz_date[32+1] = {"\0"};
    char gtm[100] = { '\0' };
    struct curl_slist *http_header = NULL;
    CURLcode res = 0;
    CURL* handler = NULL;

    // Preprocessing headers
    char header_buf[2048] = { '\0' };
    if (method_type == POST_METHOD) {
        if (headers && strlen(headers) > 0) {
            snprintf(header_buf, 1024, "%s\nContent-Type: text/plain;charset=UTF-8", headers);//;charset=UTF-8
            // snprintf(header_buf, 1024, "%s\nContent-Type: application/x-www-form-urlencoded; charset=utf-8", headers);//;charset=UTF-8
        } else {
            snprintf(header_buf, 1024, "Content-Type: text/plain;charset=UTF-8");//;charset=UTF-8
        }
        headers = header_buf;
    }
    else if (query_args && strcmp(query_args,"lifecycle") == 0 && method_type == PUT_METHOD) {
        if (headers && strlen(headers) > 0) {
            snprintf(header_buf, 1024, "%s\nContent-Type: text/plain", headers);
        } else {
            snprintf(header_buf, 1024, "Content-Type: text/plain");
        }
        if(buf_len != -1)//solve invalid argument
        {
            char cl[1024] = { '\0' };
            snprintf(cl, 1024, "Content-Length: %d", buf_len);
           // http_header = curl_slist_append(http_header, cl);
        }
        headers = header_buf;
    }
    printf("---- before make authorization headers\n %s\n---end---\n", headers);

    // make authorizations
    if(OOS_TRUE == options->is_v4){ // V2
       if (headers != header_buf){
           if (NULL == headers) {
              headers = header_buf;
	   }
	   else {
               strcpy(header_buf, headers);
               headers = header_buf;
	  }
       }
       make_V4_Authorization(options, bucket, object, headers, data, buf_len, op_type, method_type, query_args, url_encode,
           auth, date_t, gtm, x_amz_date);
    }else{  // V4
       make_V2_Authorization(options, bucket, object, headers, method_type, query_args, url_encode,
                             auth, date_t, gtm);
    }

    //  curl init
     curl_global_init(CURL_GLOBAL_ALL);
    handler = curl_easy_init();
    //  curl header append

    http_header = curl_slist_append(http_header, auth);
    if(OOS_TRUE == options->is_v4){ // V4
        //http_header = curl_slist_append(http_header, x_amz_date);
    }
    else{
        http_header = curl_slist_append(http_header, gtm);
        http_header = curl_slist_append(http_header, url_encode);
    }
    if (headers != NULL) {
        printf("add http headers:%s\n", headers);
        char *pHeadChild = strtok(headers, "\n");
        while(pHeadChild){
            printf("add child headers:%s\n", pHeadChild);
            http_header = curl_slist_append(http_header, pHeadChild);
            pHeadChild = strtok(NULL, "\n");
        }

    }

	// 6. curl set op
	curl_easy_setopt(handler, CURLOPT_SSL_VERIFYHOST, 0L);
	curl_easy_setopt(handler, CURLOPT_SSL_VERIFYPEER, 0L);
    curl_easy_setopt(handler, CURLOPT_NOSIGNAL, 1L);
    curl_easy_setopt(handler, CURLOPT_URL, url_encode);
    set_method(method_type, handler);
    curl_easy_setopt(handler, CURLOPT_HTTPHEADER, http_header);
    
    if (method_type == POST_METHOD) {
        curl_easy_setopt(handler, CURLOPT_POSTFIELDS, data);
        if (buf_len < 0)
            buf_len = 0;
        curl_easy_setopt(handler, CURLOPT_POSTFIELDSIZE, buf_len);
    }
    struct curl_slist *tmp = http_header;
    while(tmp)
    {
        printf("%http_header %s\n", tmp->data);
	    tmp = tmp->next;
    }

	// 7. curl perform
	BufData buf_data;
	do {
		if (op_type == FILE_OP) {//file_object
			file = up_file_preprocess(method_type, data, handler, resp);
			if (file == NULL) {
				break;
			}
		} else if (op_type == BUF_OP) {//buf_object
			buf_data.data = data;
			buf_data.offset = 0;
			buf_data.len = buf_len;
			buf_preprocess(method_type, &buf_data, handler, resp);
		} else if (op_type == META_OP) {//bucket
			meta_deal(handler, NULL, resp, 0);
		} else /*if (op_type == MULTI_OP || op_type == MULTI_COMPLETE_OP)*/ {
            multipart_deal(handler, NULL, 0, resp);
        }

		res = curl_easy_perform(handler);
        if (res == CURLE_OK) {
            parse_http_header(handler, resp,options->pool);
		}
	} while (0);
    //8 parser error XML
    if(NULL != resp->body && resp->status_code >=300){
        oos_parse_err_resp(resp);
    }

	// 9. curl clean: free memory, else memory leak
	curl_easy_cleanup(handler);
	if(op_type == FILE_OP && file != NULL) {	
		fclose(file);
	}
	// free memory, else memory leak
	curl_slist_free_all(http_header);
	curl_global_cleanup();
	return res;
}

void make_header(const request_options_t* options, MethodType method_type, const char* bucket,
	const char* object,const char* file_path, const char* query_args,
	const char* headers,buffer* resp, int* err) {
	*err = make_header_common(options, method_type, bucket, object,
		file_path, -1, query_args, headers, META_OP, resp);
}

void make_header_file(const request_options_t* options, MethodType method_type, const char* bucket,
	const char* object, const char* file_path, const char* query_args,
	const char* headers, buffer* resp, int* err) {
	*err = make_header_common(options, method_type, bucket, object,
		file_path, -1, query_args, headers, FILE_OP, resp);//upload_file_object, download_file_object
}

void make_header_buf(const request_options_t* options, MethodType method_type, const char* bucket,
    const char* object, const char* buf, int buf_len, const char* query_args,
    const char* headers, buffer* resp, int* err) {
    *err = make_header_common(options, method_type, bucket, object,
        buf, buf_len, query_args, headers, BUF_OP, resp);//upload_object, download_object, upload_part
}

void make_header_post(const request_options_t* options, MethodType method_type, const char* bucket,
    const char* object, const char* body, const char* query_args,
    const char* headers, buffer* resp, int* err){
    *err = make_header_common(options, method_type, bucket, object,
        body, strlen(body), query_args, headers, META_OP,  resp);//upload_object, download_object, upload_part
}

void make_multiparts(const request_options_t* options, MethodType method_type,
    const char* bucket, const char* object, const char* buf_data, int buf_len, 
    const char* query_args, const char* headers, OpType op_type,
    buffer* resp, int* err){
    *err = make_header_common(options, method_type, bucket, object,
        buf_data, buf_len, query_args, headers, op_type, resp);//init_multipart_upload, complete_multipart_upload
}

void make_lifecycle(const request_options_t* options, MethodType method_type,
                     const char* bucket, const char* object, const char* buf_data, int buf_len,
                     const char* query_args, const char* headers, 
                     buffer* resp, int* err){
    *err = make_header_common(options, method_type, bucket, object,
                              buf_data, buf_len, query_args, headers, BUF_OP, resp);
}
int make_header_presigned_url(const request_options_t* options, MethodType method_type,
    const char* bucket, const char* object, const char* data, int buf_len, const int expirs_sec,
    const char* query_args, const char* headers_str, buffer* resp, char** pre_signed_url) {
    char* headers = (char*)headers_str;

    FILE *file = NULL;
    char url_encode[1024] = {0};
    char auth[1024] = { '\0' };
    char date_t[64] = { '\0' };
    char x_amz_date[32+1] = {"\0"};
    char gtm[100] = { '\0' };
    struct curl_slist *http_header = NULL;
    CURLcode res = 0;
    CURL* handler = NULL;

    // Preprocessing headers
    char header_buf[1024] = { '\0' };
    if(headers){
        snprintf(header_buf, 1024, "%s", header_buf);//;charset=UTF-8
    }
    headers = header_buf;
    char query_param_buf[2048] = {"\0"};
    query_args = query_param_buf;

    // make authorizations
    if(OOS_TRUE == options->is_v4){ // V4
       make_V4_Authorization_presigned_url(options, bucket, object, headers, data, buf_len, method_type, query_args, url_encode,
           auth, date_t, gtm, x_amz_date, expirs_sec);
    }else{  // V2
       make_V2_Authorization_presigned_url(options, bucket, object, headers, method_type, query_args, url_encode,
                             auth, date_t, gtm, expirs_sec);
    }

    *pre_signed_url = apr_psprintf(options->pool, "%s", auth);

    resp->status_code = 200;

    return res;
}


