
#ifndef _OOS_SDK_C_API_H_
#define _OOS_SDK_C_API_H_

#ifdef __cplusplus
extern "C" {
#endif

#include <stdbool.h>
#include "response.h"
#include "list.h"
#include "oos_define.h"
#include "oos_api_define.h"
#include "oos_status.h"


/**
 *  返回所有bucket
 *
 *  @param[in] options      oos request options         
 *  @param[out] resp  查询出的oos bucket信息 
 *  @param[out] resp_header the response header

 *  @return oos_status_t
 *
 */
extern oos_status_t *oos_list_buckets(const request_options_t* options, oos_list_bucket_t **resp
                                    , oos_table_t** resp_header);

/**
 *  返回 bucket 中部分或者全部(最多 1000)的 object 信息
 *
 *  @param[in] options   oos request options 
 *  @param[in] bucket    bucket名称，长度 为 63 个字符以内，只支持小写字母数字，点(.)，以及横杠(-)
 *  @param[in] prefix    返回以prefix为前缀的结果，格式为foldername/objectprefix
 *  @param[in] delimiter 一个delimiter是一个用来对关键字们进行分组的字符。所有的关键字都包含delimiter和prefix间的相同子串，
              prefix之后第一个遇到delimiter的字符串都会加到一个叫CommonPrefix的组中。如果没有特别定义prefix，
              所有的关键字都会被返回，但不会有CommonPrefix。Delimiter只支持”/”，不支持其他分隔符。
 *  @param[in] marker    指明在bucket中list object的起始位置，Amazon S3中list object按照阿拉伯字母顺序   
 *  @param[in] max_key   指明本次获取的最大数量
 *  @param[out] resp_header the response header
 *  @param[out] objects  查询出的object信息
 *
 *  @return oos_status_t
 */
extern oos_status_t* oos_list_objects(const request_options_t* options,
                         const char* bucket, const char* prefix,const char* delimiter, 
                         const char* marker, const char* max_key, oos_list_object_t **objects, oos_table_t** resp_header);

/**
 *  创建一个新的 bucket
 *
 *  @param[in] options   oos request options 
 *  @param[in] bucket    bucket名称，长度 为 63 个字符以内，只支持小写字母数字，点(.)，以及横杠(-)
 *  @param[in] type.     设置ACL，可以为空     
 *  @param[out] resp_header the response header
 *
 *  @return oos_status_t
 */
extern oos_status_t* oos_create_bucket(const request_options_t* options,
                   const char* bucket, bucket_type_e type, struct location_configuration_t* config, oos_table_t** resp_header);
/**
 *  返回bucket的属性信息，但不会返回对象的数据内容
 *
 *  @param[in] options   oos request options
 *  @param[in] bucket    bucket名称，长度 为 63 个字符以内，只支持小写字母数字，点(.)，以及横杠(-)
 *  @param[out] resp_header the response header
 *
 *  @return oos_status_t
 */
oos_status_t* oos_head_bucket(const request_options_t* options,
                 const char* bucket, oos_table_t** resp_header);

/**
 *  删除bucket，但要求所有 bucket 中的 object 都必须被删除
 *
 *  @param[in] options   oos request options 
 *  @param[in] bucket    bucket名称，长度 为 63 个字符以内，只支持小写字母数字，点(.)，以及横杠(-)
 *  @param[out] resp_header the response header
 *
 *  @return oos_status_t
 */
extern oos_status_t* oos_delete_bucket(const request_options_t* options,
                   const char* bucket, oos_table_t** resp_header);

oos_status_t* oos_object_generate_presigned_url(const request_options_t* options,
                    const char* bucket, const char* object_key,
                    const int expires_sec,
                    oos_table_t** resp_header, char** pre_signed_url);

/**
 *  向指定 bucket 中添加一个对象
 *
 *  @param[in] options   oos request options
 *  @param[in] bucket    bucket名称，长度 为 63 个字符以内，只支持小写字母数字，点(.)，以及横杠(-)
 *  @param[in] object_key  要保存到oos的文件名称
 *  @param[in] buf_data  buffer数据
 *  @param[in] buf_len   buffer大小，单位为Byte
 *  @param[out] metadata.   需要保存到oos的元数据
 *  @param[out] resp_header the response header
 *  
 *
 *  @return oos_status_t
 *
 */
extern oos_status_t* oos_put_object_from_buffer(const request_options_t* options,
                    const char* bucket, const char* object_key,
                    const char* buf_data, int buf_len,
                     oos_table_t* metadata, oos_table_t** resp_header);

/**
 *  向指定 bucket 中添加一个对象
 *
 *  @param[in] options   oos request options
 *  @param[in] bucket    bucket名称，长度 为 63 个字符以内，只支持小写字母数字，点(.)，以及横杠(-)
 *  @param[in] file_path   要保存到的本地地址
 *  @param[in] object_key  要保存到oos的文件名称
 *  @param[out] metadata.   需要保存到oos的元数据
 *  @param[out] resp_header the response header
 *
 *  @return oos_status_t
 */
extern oos_status_t* oos_put_object_from_file(const request_options_t* options,
                     const char* bucket, const char* object_key, const char* file_path,
                   oos_table_t* metadata, oos_table_t** resp_header);

/**
 *  从指定 bucket 中下载一个对象到缓存
 *
 *  @param[in] options   oos request options
 *  @param[in] bucket    bucket名称，长度 为 63 个字符以内，只支持小写字母数字，点(.)，以及横杠(-)
 *  @param[in] object_key  待下载文件的oos路径，如foldername/objectname
 *  @param[out] metadata.   需要保存到oos的元数据
 *  @param[out] buf.     从oos获取的对象
 *  @param[out] resp_header the response header
 *
 *  @return oos_status_t
 *
 */
extern oos_status_t* oos_get_object_to_buffer(const request_options_t* options, const char* bucket,
                    const char* object_key, oos_string_t* buf,
                     oos_table_t** metadata, oos_table_t** resp_header);
/**
 *  从指定 bucket 中下载一个对象到缓存
 *
 *  @param[in] options   oos request options
 *  @param[in] bucket    bucket名称，长度 为 63 个字符以内，只支持小写字母数字，点(.)，以及横杠(-)
 *  @param[in] object_key  待下载文件的oos路径，如foldername/objectname
 *  @param[in] req_header  用户添加的额外的选项头  
 *  @param[out] metadata.   需要保存到oos的元数据
 *  @param[out] buf.     从oos获取的对象
 *  @param[out] resp_header the response header
 *
 *  @return oos_status_t
 *
 */
extern oos_status_t* oos_get_object_to_buffer_ex(const request_options_t* options, const char* bucket,
                    const char* object_key, oos_table_t* req_header, oos_string_t* buf,
                     oos_table_t** metadata, oos_table_t** resp_header);
 /**
 *  从指定 bucket 中下载一个对象，如果文件已存在则覆盖。
 *
 *  @param[in] options   oos request options
 *  @param[in] bucket    bucket名称，长度 为 63 个字符以内，只支持小写字母数字，点(.)，以及横杠(-)
 *  @param[in] object_key  待下载文件的oos路径，如foldername/objectname
 *  @param[out] metadata.   保存在oos的元数据
 *  @param[in] savepath   要保存到本地的对象的完整路径
 *  @param[out] resp_header the response header
 *
 *  @return oos_status_t
 */
extern oos_status_t* oos_get_object_to_file(const request_options_t* options,
                     const char* bucket, const char* object_key, const char* savepath,
                     oos_table_t** metadata, oos_table_t** resp_header);
/**
*  从指定 bucket 中下载一个对象，如果文件已存在则覆盖。
*
*  @param[in] options   oos request options
*  @param[in] bucket    bucket名称，长度 为 63 个字符以内，只支持小写字母数字，点(.)，以及横杠(-)
*  @param[in] object_key  待下载文件的oos路径，如foldername/objectname
*  @param[in] req_header  用户添加的额外的选项头 
*  @param[in] savepath   要保存到本地的对象的完整路径
*  @param[out] metadata.   保存在oos的元数据
*  @param[out] resp_header the response header
*
*  @return oos_status_t
*/
extern oos_status_t* oos_get_object_to_file_ex(const request_options_t* options,
	const char* bucket, const char* object_key, oos_table_t* req_header, const char* savepath,
	oos_table_t** metadata, oos_table_t** resp_header);

/**
 *  返回对象的属性信息，但不会返回对象的数据内容
 *
 *  @param[in] options   oos request options
 *  @param[in] bucket    bucket名称，长度 为 63 个字符以内，只支持小写字母数字，点(.)，以及横杠(-)
 *  @param[in] object_key  待下载文件的oos路径，如foldername/objectname
 *  @param[in] req_header  需要额外添加的请求头
 *  @param[out] resp_header the response header
 *
 *  @return oos_status_t
 */
extern oos_status_t* oos_head_object(const request_options_t* options,
                 const char* bucket, const char* object_key, oos_table_t* req_header, oos_table_t** resp_header);

/**
 *  移除指定的对象
 *
 *  @param[in] options   oos request options
 *  @param[in] bucket    bucket名称，长度 为 63 个字符以内，只支持小写字母数字，点(.)，以及横杠(-)
 *  @param[in] object_key  对象名称
 *  @param[out] resp_header the response header
 *
 *  @return oos_status_t
 */
extern oos_status_t* oos_delete_object(const request_options_t* options,
                   const char* bucket, const char* object_key, oos_table_t** resp_header);

/**
 *  批量删除对象
 *
 *  @param[in] options   oos request options
 *  @param[in] bucket    bucket名称，长度 为 63 个字符以内，只支持小写字母数字，点(.)，以及横杠(-)
 *  @param[in] object_list  对象名称列表
 *  @param[in] is_quiet  是否使用简明信息模式
 *  @param[out] resp_header the response header
 *
 *  @return oos_status_t
 */
oos_status_t* oos_delete_multiple_object(const request_options_t* options,
                   const char* bucket, delete_multiple_list_t* object_list, oos_bool is_quiet, oos_table_t** resp_header);

/**
 *  创建一个存储在 OOS 里的对象的拷贝
 *
 *  @param[in] options   oos request options
 *  @param[in] src_bucket    源bucket名称，长度 为 63 个字符以内，只支持小写字母数字，点(.)，以及横杠(-)
 *  @param[in] src_objectname   源object名称
 *  @param[in] dst_bucket    目标bucket名称，长度 为 63 个字符以内，只支持小写字母数字，点(.)，以及横杠(-)
 *  @param[in] dst_objectname   目标object名称
 *  @param[out] result   oos返回的新对象信息 
 *  @param[out] resp_header the response header
 *
 *  @return 无
 */
extern oos_status_t* oos_copy_object(const request_options_t* options, const char* src_bucket,
                 const char* src_objectname, const char* dst_bucket,
                 const char* dst_objectname,copy_object_result_t** result,
                 oos_table_t* req_header, oos_table_t** resp_header) ;


    
/**
 *  初始化一个分片上传操作
 *
 *  @param[in] options   oos request options
 *  @param[in] bucket    bucket名称，长度 为 63 个字符以内，只支持小写字母数字，点(.)，以及横杠(-)
 *  @param[in] object_key   object名称
 *  @param[in] metadata.   需要保存到oos的元数据
 *  @param[out] result     初始化分片上传的结果，包含upload ID等信息
 *  @param[out] resp_header the response header
 *
 *  @return oos_status_t
 */
extern oos_status_t* oos_init_multipart_upload(const request_options_t* options,
                           const char* bucket, const char* object_key,
                           initiate_multipart_upload_result_t** result,oos_table_t* metadata,
                           oos_table_t** resp_header);

/**
 *  上传分片
 *
 *  @param[in] options   oos request options
 *  @param[in] bucket    bucket名称，长度 为 63 个字符以内，只支持小写字母数字，点(.)，以及横杠(-)
 *  @param[in] object_key   分片所属的oos object名称
 *  @param[in] upload_id    上传ID
 *  @param[in] part_number   分片号。唯一标识一个片段并且定义该分片在对象中的位置，范围从1到10000。
                             如果用之前上传过的片段的分片号来上传新的分片，之前的分片将会被覆盖。除了最后一个分片外，
                             所有分片的大小不小于5M，最后一个分片的大小不受限制。
 *  @param[in] part_data     待上传的分片数据
 *  @param[in] part_data_len     此分片数据大小
 *  @param[out] etag.        分片上传成功后，oos返回的分片md5值 
 *  @param[out] resp_header the response header
 *
 *  @return oos_status_t
 */
extern oos_status_t* oos_upload_part_from_buffer(const request_options_t* options, const char* bucket,
                 const char* object_key, const char* upload_id,
                 int part_number, const char* part_data,int part_data_len, oos_string_t* etag,
                 oos_table_t** resp_header);
/**
 *  上传分片
 *
 *  @param[in] options   oos request options
 *  @param[in] bucket    bucket名称，长度 为 63 个字符以内，只支持小写字母数字，点(.)，以及横杠(-)
 *  @param[in] object_key   分片所属的oos object名称
 *  @param[in] upload_id    上传ID
 *  @param[in] part_number   分片号。唯一标识一个片段并且定义该分片在对象中的位置，范围从1到10000。
                             如果用之前上传过的片段的分片号来上传新的分片，之前的分片将会被覆盖。除了最后一个分片外，
                             所有分片的大小不小于5M，最后一个分片的大小不受限制。
 *  @param[in] src_bucket     拷贝源bucket名称
 *  @param[in] src_object_key     拷贝源object名称
 *  @param[in] req_header the req header
 *  @param[out] etag.        分片上传成功后，oos返回的分片md5值
 *  @param[out] resp_header the response header
 *
 *  @return oos_status_t
 */
oos_status_t* oos_copy_part(const request_options_t* options, const char* bucket,
                 const char* object_key, const char* upload_id,
                 int part_number, const char* src_bucket,const char* src_object_key,
                            oos_table_t* req_header, oos_string_t* etag,
                 oos_table_t** resp_header);
    
/**
 *  合并且完成分片上传
 *
 *  @param[in] options   oos request options
 *  @param[in] bucket    bucket名称，长度 为 63 个字符以内，只支持小写字母数字，点(.)，以及横杠(-)
 *  @param[in] object_key   object名称
 *  @param[in] upload_id    上传ID
 *  @param[in] list         片段列表，必须确保所有片段是已经上传完成的
 *  @param[in] count        片段列表中的片段个数
 *  @param[out] resp_header the response header
 *
 *  @return oos_status_t
 */
extern oos_status_t* oos_complete_multipart_upload(const request_options_t* options,
                               const char* bucket, const char* object_key,
                               const char* upload_id, part_etags list[], 
                               int count,oos_table_t** resp_header);
    
/**
 *  终止分片上传
 *  终止后，用户不能再通过上传ID上传其它片段，之前已上传完成的片段所占用的存储空间将被释放。
 *  如果此时任何片段正在上传，该上传过程可能会也可能不会成功。所以，为了释放所有片段所占用的存储空间，可能需要多次终止分片上传操作。
 *
 *  @param[in] options   oos request options
 *  @param[in] bucket    bucket名称，长度 为 63 个字符以内，只支持小写字母数字，点(.)，以及横杠(-)
 *  @param[in] object_key    object名称
 *  @param[in] upload_id     上传ID
 *  @param[out] resp_header the response header
 *
 *  @return oos_status_t
 */
extern oos_status_t* oos_abort_multipart_upload(const request_options_t* options,
                            const char* bucket, const char* object_key,
                            const char* upload_id,oos_table_t** resp_header);

/**
  该接口用于列出所有已经通过Initiate Multipart Upload请求初始化，但未完成或未终止的分片上传过程。
  响应中最多返回1000个分片上传过程的信息，它既是响应能返回的最大分片上传过程数目，
  也是请求的默认值。用户也可以通过设置max-uploads参数来限制响应中的分片上传过程数目。
  如果当前的分片上传过程数超出了这个值，则响应中会包含一个值为true的IsTruncated元素。
  如果用户要列出多于这个值的分片上传过程信息，则需要继续调用List Multipart Uploads请求，
  并在请求中设置key-marker和upload-id-marker参数。
  在响应体中，分片上传过程的信息通过key来排序。如果用户的应用程序中启动了多个使用同一key对象开头的分片上传过程，
  那么响应体中分片上传过程首先是通过key来排序，在相同key的分片上传内部则是按上传启动的起始时间的升序来进行排列
*  
*  @param[in] options   oos request options
*  @param[in] bucket    bucket名称，长度 为 63 个字符以内，只支持小写字母数字，点(.)，以及横杠(-)
*  @param[in] prefix    返回以prefix为前缀的结果，格式为foldername/objectprefix
*  @param[in] delimiter 一个delimiter是一个用来对关键字们进行分组的字符。所有的关键字都包含delimiter和prefix间的相同子串，prefix之后第一个遇到delimiter的字符串都会加到一个叫CommonPrefix的组中。如果没有特别定义prefix，所有的关键字都会被返回，但不会有CommonPrefix。Delimiter只支持”/”，不支持其他分隔符。
*  @param[in] max_uploads    设置返回的分片上传过程的最大数目，范围从1 到 1000，1000 是响应体中能返回的分片上传信息的最大值
*  @param[in] key_marker    与 upload-id-marker 参数一起，该参数指定列表操作从什么位置后面开始。如果upload-id-marker 参数没有被指定，那么只有比 key-marker参数指定的 key 更大的key会被列出。如果 upload-id-marker 参数被指定，任何包含与 key-marker 指定值相等或大于 key 的分片上传过程都会被包括，假设这些分片上传过程包含了比upload-id-marker 指定值更大的 ID
*  @param[in] upload_id_marker   与 key-marker 参数一起，指定列表操作从什么位置后面开始。如果 key-marker 没有被指定，upload-id-marker 参数也会被忽略。否则，任何 key 值等于或大于 key-marker 的上传过程都会被包含在列表中，只要 ID 大于upload-id-marker 指定的值。
*  @param[out] result   oos 返回的所有分片结果
*  @param[out] resp_header the response header
*
*  @return oos_status_t  
*/
extern oos_status_t* oos_list_multipart_uploads(const request_options_t* options, const char* bucket, const char *prefix,
	                      const char *delimiter, const char *max_uploads, const char *key_marker, const char *upload_id_marker,
                          oos_list_multipart_uploads_t** result,oos_table_t** resp_header);
    
/**
 *  列出已完成上传的所有片段
 *
 *  @param[in] options   oos request options
 *  @param[in] bucket    bucket名称，长度 为 63 个字符以内，只支持小写字母数字，点(.)，以及横杠(-)
 *  @param[in] object_key        object名称
 *  @param[in] upload_id         上传ID
 *  @param[in] maxparts          返回的片段数。一次最多能返回1000个片段信息，默认是1000个
 *  @param[in] partnumbermarker  起始片段的分片号，只列出此片段号之后(不包括此片段)的片段信息
 *                               如果有更多的片段存在，则片段列表操作没有完成，响应体中将会包含值为true的IsTruncated元素。
                                 同时，响应体中还会包含NextPartNumberMarker元素，这个值用来指定在下一次的列表操作中part-number-marker参数的值
 *  @param[out] listing          返回的所有分片信息
 *  @param[out] resp_header the response header
 *
 *  @return oos_status_t
 */
extern oos_status_t* oos_list_parts(const request_options_t* options,
                const char* bucket, const char* object_key,
                const char* upload_id, const char* maxparts,
                const char* partnumbermarker, part_listing_t** listing
                ,oos_table_t** resp_header);
    
/**
 *  设置bucket的生命周期
 *
 *  @param[in] options   oos request options
 *  @param[in] bucket    bucket名称，长度 为 63 个字符以内，只支持小写字母数字，点(.)，以及横杠(-)
 *  @param[in] config        生命周期的配置，最多一百条，会覆盖原有的生命周期
 *  @param[in] count         配置的条数
 *  @param[in] access_key    curl错误码
 *  @param[out] resp_header the response header
 *
 *  @return oos_status_t
 */
extern oos_status_t* oos_put_bucket_lifecycle(const request_options_t* options, 
                          const char* bucket, lifecycle_configuration_t config[], int count
                          ,oos_table_t** resp_header);

/**
 *  返回配置的 bucket 生命周期
 *
 *  @param[in] options   oos request options
 *  @param[in] bucket    bucket名称，长度 为 63 个字符以内，只支持小写字母数字，点(.)，以及横杠(-)
 *  @param[out] config   oos 返回的配置信息 
 *  @param[out] resp_header the response header
 *
 *  @return oos_status_t
 */
extern oos_status_t* oos_get_bucket_lifecycle(const request_options_t* options,
                          const char* bucket,lifecycle_configuration_t** config
                          ,oos_table_t** resp_header);

/**
 *  删除配置的 bucket 生命周期，用户的对象将永远不会到期，OOS 也不会再自动删除对象
 *
 *  @param[in] options   oos request options
 *  @param[in] bucket    bucket名称，长度 为 63 个字符以内，只支持小写字母数字，点(.)，以及横杠(-)
 *  @param[out] resp_header the response header
 *
 *  @return oos_status_t
 */
extern oos_status_t* oos_delete_bucket_lifecycle(const request_options_t* options, 
                             const char* bucket,oos_table_t** resp_header);

// -----------------------2018-10-30  新定义 -----------------------------------------------
/**
* WebSite 功能可以让用户将静态网站存放到 OOS 上。对于已经设置了 WebSite 的 Bucket，
* 当用户访问 http://bucketName.oos-website-cn.oos.ctyunapi.cn 时，会跳转到用户指定的主页，当出现 4**错误时，
* 会跳转到用户指定的出错页 面。如果 bucket 已经 存在了 website，
* 此操作会替换原有 website。只有 bucket 的 owner 才能执行此操 作，否则会返回 403 AccessDenied 错误。
*  @param[in] options   oos request options
*  @param[in] bucket    bucket名称，长度 为 63 个字符以内，只支持小写字母数字，点(.)，以及横杠(-)
*  @param[in] config    website 配置
*  @param[out] resp_header the response header
*
*  @return oos_status_t
*/
extern oos_status_t* oos_put_bucket_website(const request_options_t* options,
                        const char* bucket, website_configuration_t* config
                        ,oos_table_t** resp_header);

/**
*   获得指定 bucket 的 website 
*  @param[in] options   oos request options
*  @param[in] bucket    bucket名称，长度 为 63 个字符以内，只支持小写字母数字，点(.)，以及横杠(-)
*  @param[out] config   website 配置
*  @param[out] resp_header the response header
*
*  @return oos_status_t
*/
extern oos_status_t* oos_get_bucket_website(const request_options_t* options,
                        const char* bucket,website_configuration_t** config
                        ,oos_table_t** resp_header);

/**
*  删除指定 bucket 的 website。只有 bucket 的 owner 才能执行此操作，否则会返回 403 AccessDenied 错误。
*  如果 bucket 没有 website，返回 200 OK。
*  @param[in] options   oos request options
*  @param[in] bucket    bucket名称，长度 为 63 个字符以内，只支持小写字母数字，点(.)，以及横杠(-)
*  @param[out] resp_header the response header
*
*  @return oos_status_t
*/
extern oos_status_t* oos_delete_bucket_website(const request_options_t* options,
                           const char* bucket,oos_table_t** resp_header);

/**
*  如果 bucket 已经存在了 logging，此操作会替换原有 logging。只有 bucket 的 owner 才能执行此操作，否则会返回 403 AccessDenied 错误
*  @param[in] options   oos request options
*  @param[in] bucket    bucket名称，长度 为 63 个字符以内，只支持小写字母数字，点(.)，以及横杠(-)
*  @param[in] config    logging 配置。详见logging_configuration_t结构体注释
*  @param[out] resp_header the response header
*
*  @return oos_status_t
* 
*/
extern oos_status_t* oos_put_bucket_logging(const request_options_t* options,
                       const char* bucket, logging_configuration_t* config
                       ,oos_table_t** resp_header);

/**
*  获得指定 bucket 的 logging
*  @param[in] options   oos request options
*  @param[in] bucket    bucket名称，长度 为 63 个字符以内，只支持小写字母数字，点(.)，以及横杠(-)
*  @param[in] config.   logging 配置。详见logging_configuration_t结构体注释
*  @param[out] resp_header the response header
*
*  @return oos_status_t
*/
extern oos_status_t* oos_get_bucket_logging(const request_options_t* options,
                        const char* bucket, logging_configuration_t** config
                        ,oos_table_t** resp_header);

/**
*  Bucket policy 用于定义 OOS 资源的访问权限。 Policy 可以用于:
*  1 允许/拒绝 bucket 级别的权限
*  2 设置 bucket 中任何 object 的拒绝权限
*  3 为 bucket 中的 object 授权
*  Policy 本身是一个 JSON 字符串，一个 policy 包括:
*  * 可选的 policy 基本信息 
*  * 一个或多个独立的 statements
*  每个 statement 包括一个权限的核心信息。如果一个 policy 包括多个 statements,那么 statements 之间是逻辑或关系。
*   1  Version     Version 是 policy 语言的版本，它是个可选项，目前只允许填写 2018-10-17。
*   2  Id          Id 是 policy 的一个可选标识。我们推荐使用 UUID 来指明 policy。
*   3  Statement   Statement 是个重要的元素，它可以包含多个元素。Statement 元素可以包含 一组独立的 statements，
*                     每个独立的 Statement 是一个包含在大括号({})内的严 格的 JSON 块。
*                     1 Sid : Sid(statement ID)是 statement 的一个可选标识，它可以看作是 policy id 的子id。
*                     2 Effect : Effect 是一个必填元素，用于表示允许访问或拒绝访问，有效值只能是 Allow 或 Deny。
*                     3 Principal : Principal 用于指定被允许或被拒绝访问的用户，在本版本中，principal 只能是 AWS:* 或 AWS:"*" 或 AWS:["*"]，
*                       表示所有用户，暂不支持设置某个用户 ID。
*                     4 Action : Action 用于指定被允许或拒绝访问的操作，在版本中，Action 只能是"s3:*" 或 ["s3:*"]，表示所有 OOS 操作。
*                     5 Resource : Resource 是指 policy 覆盖的对象或对象集合。可以使用通配符*和?，必须 以 arn:aws:s3:::bucketName/开头。
*                       例如要使 Policy 对于所有以 image 开头的 Object 生效，可以这样配置: "Resource":"arn:aws:s3:::example-bucket/image*"
*                     6 Condition : Condition 是 Statement 中最复杂的一个元素，我们把它称之为 condition 块， 虽然只有一个 condition 元素，
*                       但是它可以包含多个 conditions，而且每个 condition 可以包含多个 key-value 对。
*                       condition 块中的各个 condition 之间是逻辑与关系，condition key 之间也是逻辑与关系，各个 value 之间是逻辑 或关系。
*  @param[in] options   oos request options
*  @param[in] bucket    bucket名称，长度 为 63 个字符以内，只支持小写字母数字，点(.)，以及横杠(-)
*  @param[in] policy_text policy 文本
*  @param[out] resp_header the response header
*
*  @return oos_status_t
*/
extern oos_status_t* oos_put_bucket_policy(const request_options_t* options,
                       const char* bucket, char* policy_text,oos_table_t** resp_header);

/**
*  获得指定 bucket 的 policy
*  @param[in] options   oos request options
*  @param[in] bucket    bucket名称，长度 为 63 个字符以内，只支持小写字母数字，点(.)，以及横杠(-)
*  @param[out] policy_text policy 文本
*  @param[out] resp_header the response header
*
*  @return oos_status_t
*/
extern oos_status_t* oos_get_bucket_policy(const request_options_t* options,
                       const char* bucket, char* policy_text,oos_table_t** resp_header);

/**
*  删除指定 bucket 的 policy只有 bucket 的 owner 才能执行此操作，否则会返回 403 AccessDenied 错误。
*  如果 bucket 没有 policy，返回 204 NoContent。
*
*  @param[in] options   oos request options
*  @param[in] bucket    bucket名称，长度 为 63 个字符以内，只支持小写字母数字，点(.)，以及横杠(-)
*  @param[out] resp_header the response header
*
*  @return oos_status_t
*/
extern oos_status_t* oos_delete_bucket_policy(const request_options_t* options,
                          const char* bucket,oos_table_t** resp_header);

/**
*  创建一对普通的 AccessKey 和 SecretKey，默认的状态是 Active。只有主 key 才能执行此操作。  
*  为保证账户的安全，SecretKey 只在创建的时候会被显示。请把 key 保存起 来，比如保存到一个文本文件中。
*  如果 SecretKey 丢失了，你可以删除 AccessKey， 并创建一对新的 key。默认情况下，每个账号最多创建 10 个 AccessKey。
*  @param[in] options   oos request options
*  @param[in] access_member  生成的 AccessKey 
*  @param[out] resp_header the response header
*
*  @return oos_status_t
*/
extern oos_status_t* oos_create_ctyun_access_key(const request_options_t* options,
                            oos_list_access_key_member_t** access_member,oos_table_t** resp_header);
/**
*  删除一对普通的 AccessKey 和 SecretKey。只有主 key 才能执行此操作。
*  @param[in] options   oos request options
*  @param[in] deleted_accessKey   将要删除的KEY
*  @param[out] resp_header the response header
*
*  @return oos_status_t
*/
extern oos_status_t* oos_ctyun_delete_access_key(const request_options_t* options,
                             const char* deleted_accessKey,oos_table_t** resp_header);

/**
*  更新普通的 AccessKey 的状态，或将普通 key 设置成为主 key，反之亦然。 只有主 key 才能执行此操作。
*  @param[in] options   oos request options
*  @param[in] updated_accessKey       将要更新的key
*  @param[in] Status                  更新后key的状态。启用或禁止。值是 Active, Inactive
*  @param[in] IsPrimary               是否更新为主key 。值是 true，false 。字符型。 
*  @param[out] resp_header the response header
*
*  @return oos_status_t
*/
extern oos_status_t* oos_ctyun_update_access_key(const request_options_t* options,
                             const char* updated_accessKey,const char* status,
                             const char* is_primary,oos_table_t** resp_header);

/**
*  列出账号下的主 ke 和普通 key。只有主 key 才能执行此操作。可以通过 MaxItems 参数指定返回的结果数量，
*  默认返回 100 个 key。可以通过 Marker 参 数设置返回的起始位置，该参数可以从前一次请求的响应体中获得。
*  为保证账号 安全，list 操作时，不会返回 SecretKey。
*  @param[in] options   oos request options
*  @param[in] MaxItems     设置返回结果集的最大数量，如果实际的 key 的数量超 过了 MaxItem，
*                          那么在响应结果中，IsTruncated 的值是 true。该参数可选。
*  @param[in] marker       可以通过 Marker 参数设置返回的起始位置，该参数可以从前一次请求的响应体中获得。该参数可选。
*  @param[out] keys     oos返回的所有key信息
*  @param[out] resp_header the response header
*
*  @return oos_status_t
*/
extern oos_status_t* oos_ctyun_list_access_key(const request_options_t* options,
                           int maxItoos_status_tems, const char* marker,oos_list_access_key_t** keys
                           ,oos_table_t** resp_header);

/**
 *  返回 Regions
 *
 *  @param[in] options   oos request options
 *  @param[out] config   oos 返回的配置信息
 *  @param[out] resp_header the response header
 *
 *  @return oos_status_t
 */
oos_status_t* oos_get_regions(const request_options_t* options,
                          region_configuration_t** config
                          ,oos_table_t** resp_header);



/**
 *  返回配置的 bucket location
 *
 *  @param[in] options   oos request options
 *  @param[in] bucket    bucket名称，长度 为 63 个字符以内，只支持小写字母数字，点(.)，以及横杠(-)
 *  @param[out] config   oos 返回的配置信息
 *  @param[out] resp_header the response header
 *
 *  @return oos_status_t
 */
extern oos_status_t* oos_get_bucket_location(const request_options_t* options,
                          const char* bucket,location_configuration_t** config
                          ,oos_table_t** resp_header);

/**
 *  返回配置的 bucket acl
 *
 *  @param[in] options   oos request options
 *  @param[in] bucket    bucket名称，长度 为 63 个字符以内，只支持小写字母数字，点(.)，以及横杠(-)
 *  @param[out] config   oos 返回的配置信息
 *  @param[out] resp_header the response header
 *
 *  @return oos_status_t
 */
oos_status_t* oos_get_bucket_acl(const request_options_t* options,
                          const char* bucket, acl_configuration_t** config
                          ,oos_table_t** resp_header);

/**
 *  设置bucket的accelerate
 *
 *  @param[in] options   oos request options
 *  @param[in] bucket    bucket名称，长度 为 63 个字符以内，只支持小写字母数字，点(.)，以及横杠(-)
 *  @param[in] config    设置的配置信息
 *  @param[out] resp_header the response header
 *
 *  @return oos_status_t
 */
oos_status_t* oos_put_bucket_accelerate(const request_options_t* options,
                          const char* bucket, accelerate_configuration_t* config
                          ,oos_table_t** resp_header);


/**
 *  返回配置的 bucket accelerate
 *
 *  @param[in] options   oos request options
 *  @param[in] bucket    bucket名称，长度 为 63 个字符以内，只支持小写字母数字，点(.)，以及横杠(-)
 *  @param[out] config   oos 返回的配置信息
 *  @param[out] resp_header the response header
 *
 *  @return oos_status_t
 */
oos_status_t* oos_get_bucket_accelerate(const request_options_t* options,
                          const char* bucket, accelerate_configuration_t** config
                          ,oos_table_t** resp_header);
/**
 *  设置bucket的cors
 *
 *  @param[in] options   oos request options
 *  @param[in] bucket    bucket名称，长度 为 63 个字符以内，只支持小写字母数字，点(.)，以及横杠(-)
 *  @param[in] config    设置的配置信息
 *  @param[out] resp_header the response header
 *
 *  @return oos_status_t
 */
oos_status_t* oos_put_bucket_cors(const request_options_t* options,
                          const char* bucket, cors_configuration_t* config
                          ,oos_table_t** resp_header);
/**
 *  返回配置的 bucket cors
 *
 *  @param[in] options   oos request options
 *  @param[in] bucket    bucket名称，长度 为 63 个字符以内，只支持小写字母数字，点(.)，以及横杠(-)
 *  @param[out] config   oos 返回的配置信息
 *  @param[out] resp_header the response header
 *
 *  @return oos_status_t
 */
oos_status_t* oos_get_bucket_cors(const request_options_t* options,
                          const char* bucket,
                          cors_configuration_t** config
                          ,oos_table_t** resp_header);

/**
 *  删除配置的 bucket cors
 *
 *  @param[in] options   oos request options
 *  @param[in] bucket    bucket名称，长度 为 63 个字符以内，只支持小写字母数字，点(.)，以及横杠(-)
 *  @param[out] resp_header the response header
 *
 *  @return oos_status_t
 */
oos_status_t* oos_delete_bucket_cors(const request_options_t* options,
                             const char* bucket,oos_table_t** resp_header);

extern void parse_meta_data(const oos_pool_t *pool, const char *headers, oos_table_t** meta_data);

#ifdef __cplusplus
}
#endif
#endif  // _KS3_SDK_C_API_H_

